﻿using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Drawing;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Web;

namespace MyCustomControls
{
    [DefaultProperty("Value")]
    [ToolboxData("<{0}:PercentageControl runat=server></{0}:PercentageControl>")]
    [ToolboxBitmap(typeof(PercentageControl), "Percentage.bmp")]
    public class PercentageControl : WebControl, IPostBackDataHandler, IScriptControl
    {
        #region Simple Control
        [Bindable(true)]
        [Category("Custom Category")]
        [DefaultValue("")]
        [Description("The percentage value.")]
        [Localizable(false)]
        public double? Value
        {
            get
            {
                var value = this.ViewState["Value"];
                if (value != null)
                {
                    return (double)value;
                }

                return default(double?);
            }

            set { this.ViewState["Value"] = value; }
        }

        public string ErrorCssClass
        {
            get
            {
                string errorCssClass = this.ViewState["ErrorCssClass"] as string;
                return errorCssClass ?? string.Empty;
            }

            set { this.ViewState["ErrorCssClass"] = value; }
        }

        protected override HtmlTextWriterTag TagKey
        {
            get { return HtmlTextWriterTag.Input; }
        }

        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            base.AddAttributesToRender(writer);
            writer.AddAttribute(HtmlTextWriterAttribute.Type, "text");
            writer.AddAttribute(HtmlTextWriterAttribute.Id, this.ClientID);
            writer.AddAttribute(HtmlTextWriterAttribute.Name, this.UniqueID);
            writer.AddAttribute(HtmlTextWriterAttribute.Value, this.Value.HasValue
                                                                ? this.Value.ToString()
                                                                : string.Empty);
        }

        #endregion

        #region IPostBackDataHandler Members

        public bool LoadPostData(string postDataKey, NameValueCollection postCollection)
        {
            var result = postCollection[this.UniqueID];

            if (string.IsNullOrEmpty(result))
            {
                this.Value = default(double?);
                return !this.Value.HasValue;
            }

            double newValue;
            if (double.TryParse(result, out newValue))
            {
                bool hasChanged = this.Value != newValue;
                this.Value = newValue;
                return hasChanged;
            }

            return false;
        }

        public void RaisePostDataChangedEvent()
        {
        }

        #endregion

        protected override void OnPreRender(System.EventArgs e)
        {
            if (!this.DesignMode)
            {
                ScriptManager sm = ScriptManager.GetCurrent(Page);
                if (sm == null)
                {
                    throw new HttpException("A ScriptManager control must exist on the page.");
                }
                sm.RegisterScriptControl(this);
            }
            base.OnPreRender(e);
            this.Page.RegisterRequiresPostBack(this);
        }

        protected override void Render(HtmlTextWriter writer)
        {
            if (!this.DesignMode)
            {
                ScriptManager sm = ScriptManager.GetCurrent(Page);
                if (sm == null)
                {
                    throw new HttpException("A ScriptManager control must exist on the page.");
                }
                sm.RegisterScriptDescriptors(this);
            }
            base.Render(writer);
        }

        #region IScriptControl

        public IEnumerable<ScriptDescriptor> GetScriptDescriptors()
        {
            ScriptControlDescriptor descriptor = new ScriptControlDescriptor("MyCustomControls.PercentageControl", this.ClientID);
            descriptor.AddProperty("errorCssClass", this.ErrorCssClass);
            return new ScriptDescriptor[] { descriptor };
        }

        public IEnumerable<ScriptReference> GetScriptReferences()
        {
            ScriptReference reference = new ScriptReference();
            if (this.Page != null)
            {
                reference.Path = this.Page.ClientScript.GetWebResourceUrl(
                                                this.GetType(),
                                                "MyCustomControls.PercentageControl.js");
            }
            return new[] { reference };
        }

        #endregion
    }
}
